package geniusweb.partiesserver.repository;

import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

import java.util.LinkedList;
import java.util.List;

import org.junit.Test;

import geniusweb.actions.PartyId;
import tudelft.utilities.logging.Reporter;
import tudelft.utilities.repository.NoResourcesNowException;

public class RunningPartiesRepoTest {
	@Test
	public void smokeTest() {
		new RunningPartiesRepo(mock(Reporter.class));
	}

	@Test
	public void testSimpleParallelPuts() throws InterruptedException {
		RunningPartiesRepo repo = new RunningPartiesRepo(mock(Reporter.class));

		List<SimpleThread> threads = new LinkedList<SimpleThread>();
		for (int n = 0; n < RunningPartiesRepo.MAX_SLOTS; n++) {
			RunningParty newParty = mock(RunningParty.class);
			when(newParty.getID()).thenReturn(new PartyId("party" + n));

			SimpleThread thread = new SimpleThread() {

				@Override
				public void run1() throws NoResourcesNowException {
					try {
						Thread.sleep(30);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					repo.put(newParty);
				}
			};
			threads.add(thread);
			thread.start();
		}

		for (SimpleThread thread : threads) {
			thread.join();
			assertNull(thread.getException());
		}

	}

	@Test
	public void testTooManyParallelPuts() throws InterruptedException {
		RunningPartiesRepo repo = new RunningPartiesRepo(mock(Reporter.class));

		List<SimpleThread> threads = new LinkedList<SimpleThread>();
		for (int n = 0; n < RunningPartiesRepo.MAX_SLOTS + 1; n++) {
			RunningParty newParty = mock(RunningParty.class);
			when(newParty.getID()).thenReturn(new PartyId("party" + n));

			SimpleThread thread = new SimpleThread() {

				@Override
				public void run1()
						throws InterruptedException, NoResourcesNowException {
					Thread.sleep(30);
					repo.put(newParty);
				}
			};
			threads.add(thread);
			thread.start();
		}

		for (SimpleThread thread : threads) {
			thread.join();
		}

		assertTrue("One of the threads should have bounced", threads.stream()
				.anyMatch(thread -> thread.getException() != null));

	}

}

/**
 * Thread that stores exceptions so that we can later check what happened.
 * Override {@link #run1()} instead of {@link #run()}
 */
abstract class SimpleThread extends Thread {
	private Exception exception = null;

	@Override
	public void run() {
		try {
			run1();
		} catch (Exception e) {
			this.exception = e;
			e.printStackTrace();
		}
	}

	public Exception getException() {
		return exception;
	}

	abstract void run1() throws Exception;
}
